`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : mb_mcs_sys.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze SD_CARD system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2014/01/23
//*****************************************************************************

module mb_mcs_sys(  Clk, Reset, UART_Rx, UART_Tx,led,dip_sw,
                    SCLK,MISO,MOSI,SS_n,
                    mon
    );

  input  Clk;      //input clock 100MHz
  input  Reset;    //reset Hi=reset
  input  UART_Rx;  //UART rx 115200bps
  output UART_Tx; //UART tx 115200bps
  output [3 : 0] led;   //led Hi=on
  input  [3 : 0] dip_sw; //input dip_sw
  output         SCLK ; //spi clk
  input          MISO ; //spi master input data
  output         MOSI ; //spi master output data
  output         SS_n ;   //spi select
  output         mon;
   
  // 
  parameter p_reg_addr_low =  32'hC0000000;
  parameter p_reg_addr_hi  =  32'hC0000007;
  parameter p_bram_addr_low = 32'hC0001000;
  parameter p_bram_addr_hi  = 32'hC00017FF;
  // 
  parameter p_wb_addr_low  =  32'hD0000000;
  parameter p_wb_addr_hi   =  32'hEFFFFFFF;
  parameter p_wb_pwm0_low   =  p_wb_addr_low + 0;
  parameter p_wb_pwm0_hi    =  p_wb_addr_low + 15;
  parameter p_wb_pwm1_low   =  p_wb_addr_low + 16;
  parameter p_wb_pwm1_hi    =  p_wb_addr_low + 31;
  parameter p_wb_spi_low   =  p_wb_addr_low + 256;
  parameter p_wb_spi_hi    =  p_wb_addr_low + 511;
  //IO Bus
  wire IO_Addr_Strobe;
  wire IO_Read_Strobe;
  wire IO_Write_Strobe;
  wire [31 : 0] IO_Read_Data;
  wire [31 : 0] IO_Read_Data_reg;
  wire [31 : 0] IO_Read_Data_bram;
  wire [31 : 0] IO_Read_Data_wb;
  wire [31 : 0] IO_Address;
  wire [3 : 0]  IO_Byte_Enable;
  wire [31 : 0] IO_Write_Data;
  wire IO_Ready;
  wire IO_Ready_reg;
  wire IO_Ready_bram;
  wire IO_Ready_wb;
  wire iobus_reg_sel;
  wire iobus_bram_sel;  
  wire iobus_wb_sel;   
  //timmer
  wire FIT1_Interrupt;
  wire FIT1_Toggle;    
  wire PIT1_Interrupt; 
  wire PIT1_Toggle; 
  //GPO   
  wire [31 : 0] GPO1;
  wire [31 : 0] GPI1;
  //GPI
  wire GPI1_Interrupt;
  //Interrupt controller
  wire INTC_Interrupt;
  wire INTC_IRQ;
  //iobus_reg
  wire [31:0] data_in;
  wire [3:0]  data_in_en;
  wire [31:0] data_out;
  wire [3:0]  data_out_en;
  //user_module
  wire [7:0] chk_data; 
  //block ram port-B
  wire  [3:0]web;
  wire  [8 : 0] addrb;
  wire  [31 : 0] dinb;
  wire  [31 : 0] doutb;
  //Wishbone bus microblaze -> IP
  wire         WB_CLK;         // clock 
  wire         WB_RST;         // reset (asynchronous active hi)
  wire         WB_CYC;         // cycle
  wire         WB_STB;         // strobe
  wire  [31:0] WB_ADR;         // address adr_i
  wire         WB_WE;          // write enable
  wire  [31:0] WB_DAT_I;          // data output
  wire  [31:0] WB_DAT_O;          // data input
  wire  [3:0]  WB_SEL;         // byte enable
  wire         WB_ACK;         // normal bus termination
  wire         WB_ERR;         // err bus termination
  wire         WB_RTY;         // rtry
  //pwm
  wire         cyc_i_pwm0;         // cycle
  wire         stb_i_pwm0;         // strobe
  wire         we_i_pwm0;          // write enable
  wire  [15:0] adr_i_pwm0;         // address adr_i
  wire  [15:0] dat_o_pwm0;   // data input
  wire         ack_o_pwm0;         // normal bus termination
  wire         wb_pwm0_sel;
  wire         cyc_i_pwm1;         // cycle
  wire         stb_i_pwm1;         // strobe
  wire  [15:0] adr_i_pwm1;         // address adr_i
  wire         we_i_pwm1;          // write enable
  wire  [15:0] dat_o_pwm1;   // data input
  wire         ack_o_pwm1;         // normal bus termination
  wire         wb_pwm1_sel;
  //spi
  wire         cyc_i_spi;         // cycle
  wire         stb_i_spi;         // strobe
  wire         we_i_spi;          // write enable
  wire  [7:0] dat_o_spi;   // data input
  wire         ack_o_spi;         // normal bus termination
  wire         wb_spi_sel;
   
  //opencores PWM signals
  wire  i_extclk;
  wire  [15:0]i_DC;
  wire  i_valid_DC;  
  wire  o_pwm0;   
   
  //opencores SPI signals
  wire spiClkOut;
  wire spiDataIn;
  wire spiDataOut;
  wire spiCS_n;

  wire clk_100m;
  wire clk_50m;
  wire clk_20m;
  wire clk_10m;
  wire RESET;
  wire LOCKED;
  //
   
  reg         SCLK_mon ; //spi clk
  reg         MISO_mon ; //spi master input data
  reg         MOSI_mon ; //spi master output data
  reg         SS_n_mon ;   //spi select
  reg         mon;

  clk_gen clk_gen
   (// Clock in ports
    .CLK_IN1(Clk),      // IN
    // Clock out ports
    .CLK_OUT1(Clk_i),     // OUT
    .CLK_OUT2(Clk_50m),     // OUT
    .CLK_OUT3(clk_20m),     // OUT
    .CLK_OUT4(clk_10m),     // OUT
    // Status and control signals
    .RESET(RESET),// IN
    .LOCKED(LOCKED));      // OUT
   //   
  //LED$B@)8f(B
  
  //assign led = {data_out[1:0],GPO1[1:0]};
  //assign led = {2'b00,o_pwm1,o_pwm0};
  assign led = {data_out[1:0],o_pwm1,o_pwm0};
  //$B?.9f@\B3(B
  assign GPI1 = {28'h0000000,dip_sw};
  assign data_in = data_out;
  assign data_in_en = data_out_en; 
  assign chk_data = data_out[7:0];  
  assign INTC_Interrupt = 1'b0;  

  //$B<~JU2sO)A*Br?.9f(B
  assign iobus_reg_sel  = ((IO_Address>=p_reg_addr_low)&&(IO_Address<=p_reg_addr_hi))? 1'b1:1'b0;
  assign iobus_bram_sel = ((IO_Address>=p_bram_addr_low)&&(IO_Address<=p_bram_addr_hi))? 1'b1:1'b0;   
  assign iobus_wb_sel  =  ((IO_Address>=p_wb_addr_low)&&(IO_Address<=p_wb_addr_hi))? 1'b1:1'b0;
  //$BFI=P$7%G!<%?$NA*Br(B       
  assign IO_Read_Data = (iobus_reg_sel==1'b1)?  IO_Read_Data_reg:
                        (iobus_bram_sel==1'b1)? IO_Read_Data_bram:
                        (iobus_wb_sel==1'b1)?   IO_Read_Data_wb:
                                                32'h00000000;
  //Ready$B?.9f$NA*Br(B  
  assign IO_Ready =    (iobus_reg_sel==1'b1)?  IO_Ready_reg:
                       (iobus_bram_sel==1'b1)? IO_Ready_bram:
                       (iobus_wb_sel==1'b1)?   IO_Ready_wb:
                                               (IO_Write_Strobe||IO_Read_Strobe);

//Microblaze MCS$B$N%$%s%9%?%s%9(B
mb_mcs mcs_0 (
  .Clk(Clk_i), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready), // input IO_Ready
  .UART_Rx(UART_Rx), // input UART_Rx
  .UART_Tx(UART_Tx), // output UART_Tx
  .FIT1_Interrupt(FIT1_Interrupt), // output FIT1_Interrupt
  .FIT1_Toggle(FIT1_Toggle), // output FIT1_Toggle
  .PIT1_Interrupt(PIT1_Interrupt), // output PIT1_Interrupt
  .PIT1_Toggle(PIT1_Toggle), // output PIT1_Toggle
  .GPO1(GPO1), // output [31 : 0] GPO1
  .GPI1(GPI1), // input [31 : 0] GPI1
  .GPI1_Interrupt(GPI1_Interrupt), // output GPI1_Interrupt
  .INTC_Interrupt(INTC_Interrupt), // input [0 : 0] INTC_Interrupt
  .INTC_IRQ(INTC_IRQ) // output INTC_IRQ
); 
 
//$B@_Dj%l%8%9%?$N%$%s%9%?%s%9(B
iobus_reg
#(p_reg_addr_low, p_reg_addr_hi)
iobus_reg_0 (
  .Clk(Clk_i), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_reg), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_reg), // input IO_Ready
  //input 
  .data_in(data_in), // input data
  .data_in_en(data_in_en), // input data enable
  //output 
  .data_out(data_out), // output data
  .data_out_en(data_out_en) // output data enable
);

//$B%V%m%C%/(BRAM$B%$%s%?!<%U%'!<%9$N%$%s%9%?%s%9(B
iobus_bram
 #(p_bram_addr_low, p_bram_addr_hi)
iobus_bram_0 (
  .Clk(Clk_i), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_bram), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_bram), // input IO_Ready
  //BRAM Port-B                  
  .clkb(Clk_i), // input clkb
  .web(web), // input [3 : 0] web
  .addrb(addrb), // input [8 : 0] addrb
  .dinb(dinb), // input [31 : 0] dinb
  .doutb(doutb) // output [31 : 0] doutb
);
   
user_module user_module_0 (
  .Clk(Clk_i), // input Clk
  .Reset(Reset), // input Reset
  .chk_data(chk_data),
  //BRAM Port-B                  
  .clk(clkb), // input clkb
  .we(web), // input [3 : 0] web
  .addr(addrb), // input [8 : 0] addrb
  .dout(dinb) // output [31 : 0] doutb
);

iobus2wb 
#(p_wb_addr_low, p_wb_addr_hi)
iobus2wb(
  .Clk(Clk_i), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_wb), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_wb), // input IO_Ready
  //wishbone MicorBlaze -> IP
  .CLK_O(WB_CLK),         // clock
  .RST_O(WB_RST),         // reset 
  .CYC_O(WB_CYC),         // cycle
  .STB_O(WB_STB),         // strobe
  .ADR_O(WB_ADR),         // address adr_i
  .SEL_O(WB_SEL),         // byte_sel
  .WE_O( WB_WE),          // write enable
  .DAT_O(WB_DAT_I),       // data output
  .DAT_I(WB_DAT_O),       // data input
  .ACK_I(WB_ACK),         // normal bus termination
  .ERR_I(WB_ERR),         // error bus termination
  .RTY_I(WB_RTY)          // retry
                  );
     
//opencores simple_gpio    
 PWM PWM_0(
  .i_wb_clk(WB_CLK),         // clock
  .i_wb_rst(WB_RST),        // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm0),    // cycle
  .i_wb_stb(stb_i_pwm0),    // strobe
  .i_wb_adr(adr_i_pwm0),         // address adr_i
  .i_wb_we(we_i_pwm0),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm0),    // data input
  .o_wb_ack(ack_o_pwm0),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm0)
                         ); 
 PWM PWM_1(
  .i_wb_clk(WB_CLK),       // clock
  .i_wb_rst(WB_RST),       // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm1),    // cycle
  .i_wb_stb(stb_i_pwm1),    // strobe
  .i_wb_adr(adr_i_pwm1), // address adr_i
  .i_wb_we(we_i_pwm1),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm1),   // data input
  .o_wb_ack(ack_o_pwm1),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm1)
                         );
//SPI Master
spiMaster u_spiMaster (
  // SPI system clock
  .spiSysClk(clk_20m),
  //SPI 
  .spiClkOut(spiClkOut),
  .spiDataIn(spiDataIn),
  .spiDataOut(spiDataOut),
  .spiCS_n(spiCS_n),
  //WISHBONE Bus
  .clk_i(WB_CLK),
  .rst_i(WB_RST),
  .address_i(WB_ADR[7:0]),
  .data_i(WB_DAT_I[7:0]),
  .data_o(dat_o_spi),
  .strobe_i(stb_i_spi),
  .we_i(we_i_spi),
  .ack_o(ack_o_spi)
);

//SPI signal
   assign SCLK = spiClkOut;
   assign spiDataIn = MISO;
   assign MOSI = spiDataOut;
   assign SS_n =  spiCS_n;
   
   
//WihsBoneBus interconnect
   assign wb_pwm0_sel = ((WB_ADR>=p_wb_pwm0_low)&&(WB_ADR<=p_wb_pwm0_hi))? 1'b1:1'b0;
   assign cyc_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm0 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm0  =(wb_pwm0_sel==1'b1)?WB_WE:1'b0;
   
   assign wb_pwm1_sel = ((WB_ADR>=p_wb_pwm1_low)&&(WB_ADR<=p_wb_pwm1_hi))? 1'b1:1'b0;
   assign cyc_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm1 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm1  =(wb_pwm1_sel==1'b1)?WB_WE:1'b0;
//spi
   assign wb_spi_sel = ((WB_ADR>=p_wb_spi_low)&&(WB_ADR<=p_wb_spi_hi))? 1'b1:1'b0;
   assign cyc_i_spi =(wb_spi_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_spi =(wb_spi_sel==1'b1)?WB_STB:1'b0;
   assign we_i_spi  =(wb_spi_sel==1'b1)?WB_WE:1'b0;

   assign WB_DAT_O =(wb_pwm0_sel==1'b1)?{dat_o_pwm0,dat_o_pwm0}:
                    (wb_pwm1_sel==1'b1)?{dat_o_pwm1,dat_o_pwm1}:
                    (wb_spi_sel==1'b1)?{dat_o_spi,dat_o_spi,dat_o_spi,dat_o_spi}:
                                        32'h00000000;
   
   assign WB_ACK = (wb_pwm0_sel==1'b1)?ack_o_pwm0:
                   (wb_pwm1_sel==1'b1)?ack_o_pwm1:
                   (wb_spi_sel==1'b1)?ack_o_spi:
                                      WB_STB;
   
   assign WB_ERR = 1'b0;
   assign WB_RTY = 1'b0;
   
// PWM inputs
   assign i_extclk =1'b0;
   assign i_DC = 16'h0000;
   assign i_valid_DC =1'b0;    

//mon
   
always @ (posedge clk_20m )
  mon <= SCLK_mon ^ MISO_mon ^ MOSI_mon ^ SS_n_mon;


always @ (posedge clk_20m  )
    begin
      SCLK_mon    <= SCLK;
      MISO_mon  <= MISO     ;
      MOSI_mon <= MOSI      ;
      SS_n_mon    <= SS_n     ;
    end   
   
endmodule
